#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
# Barcerlona Supercomputing Center - Centro Nacional de Supercomputacion
#        Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)
#             -
#             -
#

import sys
import logging
import os
import subprocess

class standard_compliance_test:

    def __init__(self,path,type=None):
        self.type = type
        self.path = path
        self.files = self.list_files_folder()

    def list_files_folder(self):
        list_files = []
        
        if os.path.exists(self.path) and os.path.isdir(self.path):
            for root, dirs, files in os.walk(self.path):
                    for f in files:
                        if f.endswith("grib") or f.endswith("nc"):
                            list_files.append(os.path.join(root,f))
            
            return sorted(list_files)
        else:
            raise Exception('\n\tGiven path is not a valid path\n')

    def run_cfchecker(self,item):
        cmd0 = "cfchecks --version auto %s" %item

        p0 = Popen(
            cmd0, shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
            )

        out0, _ = p0.communicate()

        return out0

    def netcdf_test(self,args):
        from subprocess import Popen

        output = list()
        for ncfile in args:

            out0 = self.run_cfchecker(ncfile)

            string = str(out0).replace('\'',"").replace("\"","").split('\\n')
            if any('ERROR:' in e for e in string):
                err = [item.split('ERROR:')[-1].strip() for item in string if item.startswith('ERROR:')]
                errstring = ncfile.split('/')[-1] + ' has ' + str(len(err))
                errstring += ' error messages' + '\n'
                errstring += '\n'.join(err)
                print(errstring)
                output.append(errstring)
            else:
                pass
            # We are only interested on errors but could be interesting for 
            # further versions of the dqc_checker to analyze the warnings 
            # and info messages
            # wrn = [item.split('WARN:')[-1].strip() for item in string if item.startswith('WARN:')]
            # inf = [item.split('INFO:')[-1].strip() for item in string if item.startswith('INFO:')]

    def grib_test(args):
        from utils.grib_checker import grib_cck

        for gribfile in args:
            grib_cck.run(gribfile)


    def main(self):
        if self.type != None:

            if self.type == 'CF':
                ncfiles = [item for item in self.files if item.endswith('.nc')]
                if ncfiles != []:
                    self.netcdf_test(ncfiles)
                else:
                    raise Exception('No NetCDF files in the given input')

            if self.type == 'grib':
                gribfiles = [item for item in self.files if item.endswith('.grib')]
                if gribfiles != []:
                    self.grib_test(gribfiles)
                else:
                    raise Exception('No GRIB files in the given input')

        else:
            sys.exit()

if __name__ == "__main__":
    sys.exit()