#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
# Barcerlona Supercomputing Center - Centro Nacional de Supercomputacion
#        Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)
#             -
#             -
#

import sys
import logging
import os
import subprocess

class spatial_completeness_test:

    def __init__(self,path,type=None):
        self.type = type
        self.path = path
        self.files = self.list_files_folder()

    def list_files_folder(self):
        list_files = []
        
        if os.path.exists(self.path) and os.path.isdir(self.path):
            for root, dirs, files in os.walk(self.path):
                    for f in files:
                        if f.endswith("grib") or f.endswith("nc"):
                            list_files.append(os.path.join(root,f))
            
            return sorted(list_files)
        else:
            raise Exception('\n\tGiven path is not a valid path\n')

    def missing_values_netcdf(self,ncitem,e):
        var = ncitem.variables[e]
        variable = var[:]
        missing = variable.missing_value
        if var.mask == False:
            # If the variable does not have a mask
            datamiss = len(variable==missing)
            mk = False if len(datamiss) == 0 else True
        else:
            # If the variable does have a mask
            data_masked = variable.mask
            data = variable.data
            real_data = data[~data_masked]
            datamiss = len(real_data==missing)
            mk = False if len(datamiss) == 0 else True

        return datamiss,mk

    def netcdf_test(self,args):
        import netCDF4
        import numpy as np

        output = list()

        for ncfile in args:
            ncitem = netCDF4.Dataset(f)
            test_var = [item for item in ncitem.variables.keys() \
                if item not in ncitem.dimensions.keys()]
            if test_var != []:
                for e in test_var:
                    datamiss,mk = self.missing_values_netcdf(ncitem,e)
                    if mk == True:
                        string = ncfile.split('/')[-1] + ' variable '
                        string += e + ' has missing data ' + datamiss
                        output.append('\n'.join([ncfile.split('/')[-1],string,e]))

        if len(output) == 0:
            print('No gaps found! Hurray!')

        else:
            print('\n\n'.join(output))

    def grib_test(self,args):
        print('WILL BE READY SOON')

    def main(self):
        if self.type != None:

            if self.type == 'netcdf':
                ncfiles = [item for item in self.files if item.endswith('.nc')]
                if ncfiles != []:
                    self.netcdf_test(ncfiles)
                else:
                    raise Exception('\n\tNo NetCDF files in the given input\n')

            if self.type == 'grib':
                gribfiles = [item for item in self.files if item.endswith('.grib')]
                if gribfiles != []:
                    self.grib_test(gribfiles)
                else:
                    raise Exception('\n\tNo GRIB files in the given input\n')

        else:
            raise Exception('\n\tStandard type has not been given by the user\n')

if __name__ == "__main__":
    sys.exit()